﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;

namespace Chocoat.DccSupport
{
    /// <summary>
    /// Form1とか、名前めんどうー
    /// </summary>
    public partial class Form1 : Form
    {
        #region constructor
        /// <summary>
        /// Form1 の新しいインスタンスを生成します。
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            this.dgvPortMapping.AutoGenerateColumns = false;
        }
        #endregion

        #region OnLoad
        /// <summary>
        /// OnLoad
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.btnReload.PerformClick();
            this.btnChocoaNat.PerformClick();
            this.btnWan.PerformClick();
        }
        #endregion

        #region OnClosing / OnClosing
        /// <summary>
        /// OnClosing
        /// </summary>
        protected override void OnClosing(CancelEventArgs e)
        {
            //これ、やっぱやめた
            Properties.Settings.Default.Save();
            base.OnClosing(e);
        }
        #endregion

        #region btnReload_Click / ちょこあの接続されているサーバーを取り直す
        /// <summary>
        /// ちょこあの接続されているサーバーを取り直す
        /// </summary>
        private void btnReload_Click(object sender, EventArgs e)
        {
            //さいけんさくー
            this.cmdServerList.Items.Clear();

            foreach (string serverName in this.compChocoat.GetConnectedServerList())
            {
                this.cmdServerList.Items.Add(serverName);
            }

            if (this.cmdServerList.Items.Count > 0)
            {
                this.cmdServerList.SelectedIndex = 0;
            }

            this.btnChocoaNat.PerformClick();
        }
        #endregion

        #region btnChocoaNat_Click / 接続されているサーバーのNAT設定を取得する
        /// <summary>
        /// 接続されているサーバーのNAT設定を取得する
        /// </summary>
        private void btnChocoaNat_Click(object sender, EventArgs e)
        {
            this.txtChocoaNat.Text = String.Empty;

            if (this.cmdServerList.SelectedIndex != -1)
            {
                int serverID = this.compChocoat.FindServerID((string)this.cmdServerList.SelectedItem);

                if (this.compChocoat.GetVariable(serverID, "$usenat") != "0")
                {
                    string ip = this.compChocoat.GetVariable(serverID, "$natip");
                    IPAddress ipAddress = Chocoat.Utility.ChocoaIPConverter.ToIPAddress(Convert.ToInt32(ip));

                    string port = this.compChocoat.GetVariable(serverID, "$natport");

                    this.txtChocoaNat.Text = String.Format("{0}:{1}", ipAddress, port);
                }
                else
                {
                    this.txtChocoaNat.Text = "使わない設定です。";
                }
            }
        } 
        #endregion

        #region btnWan_Click / WANアドレスの取り直し
        /// <summary>
        /// WANアドレスの取り直し
        /// </summary>
        private void btnWan_Click(object sender, EventArgs e)
        {
            Chocoat.Utility.UPnP p = new Chocoat.Utility.UPnP();

            this.txtWan.Text = p.GetWanAddress().ToString();

            //次いでにポートマップも取得
            DataTable table = p.GetPortMapping();

            this.dgvPortMapping.DataSource = table;
        } 
        #endregion

        #region dgvPortMapping_CellContentClick / 削除する
        /// <summary>
        /// 削除する
        /// </summary>
        private void dgvPortMapping_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex != -1 && e.ColumnIndex != -1 &&
                this.dgvPortMapping.Columns[e.ColumnIndex] == this.dataGridViewButtonColumnDelete)
            {
                DataRowView rowView = (DataRowView)this.dgvPortMapping.Rows[e.RowIndex].DataBoundItem;

                string ipString = (string)rowView.Row["RemoteAddress"];
                int port = (int)rowView.Row["NewExternalPort"];
                string protocol = (string)rowView.Row["NewProtocol"];

                IPAddress ipAddress = null;

                if (!IPAddress.TryParse(ipString, out ipAddress))
                {
                    MessageBox.Show("IPアドレスの取得に失敗しました。");
                }
                else if (MessageBox.Show("ポートマップを削除します。よろしいですか？", "確認",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                {
                    Chocoat.Utility.UPnP p = new Chocoat.Utility.UPnP();

                    bool isDeleted = false;

                    if (protocol == "UDP")
                    {
                        isDeleted = p.DeleteUdpPortMapping(ipAddress, port);
                    }
                    else
                    {
                        isDeleted = p.DeleteTcpPortMapping(ipAddress, port);
                    }

                    if (!isDeleted)
                    {
                        MessageBox.Show("なんか失敗しました。");
                    }
                    else
                    {
                        //リロード
                        this.btnWan.PerformClick();

                        if (MessageBox.Show("ついでに、ちょこあのNATを外しますか？", "確認",
                            MessageBoxButtons.YesNo,
                            MessageBoxIcon.Question) == DialogResult.Yes)
                        {
                            if (!this.compChocoat.IsStart || this.cmdServerList.SelectedIndex == -1)
                            {
                                MessageBox.Show("CHOCOAが見つからないか、サーバーが選択されていません。\r\n" +
                                    "NAT変換は追加されませんでした。");
                            }
                            else
                            {
                                int serverID = this.compChocoat.FindServerID((string)this.cmdServerList.SelectedItem);
                                this.compChocoat.SetVariable(serverID, "$usenat", "0");
                                this.compChocoat.SetVariable(serverID, "$natip", "0");
                                this.compChocoat.SetVariable(serverID, "$natport", "0");

                                this.btnChocoaNat.PerformClick();
                            }
                        }
                    }
                }
            }
        } 
        #endregion

        #region btnAddMapping_Click / ポートマップを追加する
        /// <summary>
        /// ポートマップを追加する
        /// </summary>
        private void btnAddMapping_Click(object sender, EventArgs e)
        {
            int port = (int)this.numPort.Value;
            int duration = (int)this.numLeaseDuration.Value;

            if (this.chkAutoPort.Checked)
            {
                Chocoat.Utility.TcpTable tcpTable = Chocoat.Utility.TcpTable.Current;

                while (tcpTable.AlreadyPortUse(port))
                {
                    port++;
                }

                this.numPort.Value = port;
            }

            IPAddress ipAddress = null;

            if (!IPAddress.TryParse(this.txtWan.Text, out ipAddress))
            {
                MessageBox.Show("WAN側のIPアドレス変換に失敗しました。");
            }
            else
            {
                if (MessageBox.Show("ポートを追加して、CHOCOAにNAT設定します。よろしいですか？", "確認",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    Chocoat.Utility.UPnP p = new Chocoat.Utility.UPnP();

                    bool isOpen = false;

                    if (this.rbTcp.Checked)
                    {
                        isOpen = p.AddTcpPortMapping(port, duration, this.txtDescription.TextLength == 0 ? "Chocoat Open" : this.txtDescription.Text);
                    }
                    else if (this.rbUdp.Checked)
                    {
                        isOpen = p.AddUdpPortMapping(port, duration, this.txtDescription.TextLength == 0 ? "Chocoat Open" : this.txtDescription.Text);
                    }

                    if (!isOpen)
                    {
                        MessageBox.Show("なんか失敗しました。");
                    }
                    else
                    {
                        if (!this.compChocoat.IsStart || this.cmdServerList.SelectedIndex == -1)
                        {
                            MessageBox.Show("ポートは追加しましたが、CHOCOAが見つからないか、サーバーが選択されていません。\r\n" +
                                "NAT変換は追加されませんでした。");
                        }
                        else
                        {
                            int serverID = this.compChocoat.FindServerID((string)this.cmdServerList.SelectedItem);
                            this.compChocoat.SetVariable(serverID, "$usenat", "1");
                            this.compChocoat.SetVariable(serverID, "$natip", Chocoat.Utility.ChocoaIPConverter.ToInt32(ipAddress).ToString());
                            this.compChocoat.SetVariable(serverID, "$natport", port.ToString());

                            this.btnChocoaNat.PerformClick();
                            this.btnWan.PerformClick();
                        }
                    }
                }
            }
        } 
        #endregion
    }
}
