﻿using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// CHOCOAのウィンドウ状態が変更された時に呼び出されるイベントで利用されるイベントデータです。
    /// </summary>
    [Serializable]
    public sealed class ChocoaWindowEventArgs : EventArgs
    {
        /// <summary>
        /// ChocoaWindowEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="windowState">ウィンドウ状態を取得します。詳しくはCCAPIのヘルプ</param>
        /// <param name="xtopLeft">左上X</param>
        /// <param name="yTopLeft">左上Y</param>
        /// <param name="xBottomRight">右下X</param>
        /// <param name="yBottomRight">右下Y</param>
        internal ChocoaWindowEventArgs(
            string settingServerName,
            int windowState,
            int xtopLeft,
            int yTopLeft,
            int xBottomRight,
            int yBottomRight)
        {
            this.SettingServerName = settingServerName;
            this.WindowState = windowState;

            this.Location = new Point(xtopLeft, yTopLeft);
            this.Size = new Size(xBottomRight - xtopLeft, yBottomRight - yTopLeft);
        }

        /// <summary>
        /// サーバー設定名を取得します。
        /// </summary>
        public string SettingServerName { get; private set; }

        /// <summary>
        /// ウィンドウの状態のステータスを取得します。意味する内容はCCAPIリファレンスを参照してください。
        /// </summary>
        public int WindowState { get; private set; }

        /// <summary>
        /// ウィンドウのロケーションを取得します。
        /// </summary>
        public Point Location { get; private set; }

        /// <summary>
        /// ウィンドウのサイズを取得します。
        /// </summary>
        public Size Size { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, WindowState:{1}, Location:{2}, Size:{3}",
                this.SettingServerName, this.WindowState, this.Location, this.Size);
        }
    }
}
