﻿using System;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// IRCのKickイベントを受信した際の情報を提供します。
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [Serializable]
    public sealed class IRCKickEventArgs : IRCEventArgs
    {
        /// <summary>
        /// IRCKickEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="reValue">通知内容</param>
        internal IRCKickEventArgs(string settingServerName, string reValue)
            : base(settingServerName)
        {
            string[] args = reValue.Split(' ');

            this.Nick = args[0];
            this.HostName = args[1];
            this.Channel = args[2];
            this.KickedUser = args[3];
            this.Comment = args[4];
        }

        /// <summary>
        /// ニックネームを取得します。
        /// </summary>
        public string Nick { get; private set; }

        /// <summary>
        /// ホスト名を取得します。
        /// </summary>
        public string HostName { get; private set; }

        /// <summary>
        /// チャンネルを取得します。
        /// </summary>
        public string Channel { get; private set; }

        /// <summary>
        /// 蹴り出される人を取得します。
        /// </summary>
        public string KickedUser { get; private set; }

        /// <summary>
        /// 蹴り出された理由を取得します。
        /// </summary>
        public string Comment { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, Nick:{1}, HostName:{2}, Channel:{3}, KickedUser:{4}, Comment:{5}",
                this.SettingServerName, this.Nick, this.HostName, this.Channel, this.KickedUser, this.Comment);
        }
    }
}
