﻿using System;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// IRCのNAMESイベントを受信した際の情報を提供します。
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [Serializable]
    public sealed class IRCNamesEventArgs : IRCEventArgs
    {
        /// <summary>
        /// IRCNamesEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="reValue">通知内容</param>
        internal IRCNamesEventArgs(string settingServerName, string reValue)
            : base(settingServerName)
        {
            string[] args = reValue.Split(' ');

            this.Channel = args[0];
            args.CopyTo(this.UserList, 1);
        }

        /// <summary>
        /// チャンネルを取得します。
        /// </summary>
        public string Channel { get; private set; }

        /// <summary>
        /// ユーザーリストを取得します。
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
        public string[] UserList { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, Channel:{1}, UserList:[{2}]",
                this.SettingServerName, this.Channel, this.UserList.Length);
        }
    }
}
