﻿using System;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// IRCのNickイベントを受信した際の情報を提供します。
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [Serializable]
    public sealed class IRCNickEventArgs : IRCEventArgs
    {
        /// <summary>
        /// IRCNickEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="reValue">通知内容</param>
        internal IRCNickEventArgs(string settingServerName, string reValue)
            : base(settingServerName)
        {
            string[] args = reValue.Split(' ');

            this.NewNick = args[0];
            this.OldNick = args[1];
        }

        /// <summary>
        /// 新しいニックネームを取得します。
        /// </summary>
        public string NewNick { get; private set; }

        /// <summary>
        /// 旧ニックネームを取得します。
        /// </summary>
        public string OldNick { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, NewNick:{1}, OldNick:{2}",
                this.SettingServerName, this.NewNick, this.OldNick);
        }
    }
}
