﻿using System;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// IRCのTopicイベントを受信した際の情報を提供します。
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [Serializable]
    public sealed class IRCTopicEventArgs : IRCEventArgs
    {
        /// <summary>
        /// IRCTopicEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="reValue">通知内容</param>
        internal IRCTopicEventArgs(string settingServerName, string reValue)
            : base(settingServerName)
        {
            string[] args = reValue.Split(' ');

            this.Nick = args[0];
            this.HostName = args[1];
            this.Channel = args[2];
            this.Topic = args[3];
        }

        /// <summary>
        /// ニックネームを取得します。
        /// </summary>
        public string Nick { get; private set; }

        /// <summary>
        /// ホスト名を取得します。
        /// </summary>
        public string HostName { get; private set; }

        /// <summary>
        /// チャンネルを取得します。
        /// </summary>
        public string Channel { get; private set; }

        /// <summary>
        /// トピックを取得します。
        /// </summary>
        public string Topic { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, Nick:{1}, HostName:{2}, Channel:{3}, Topic:{4}",
                this.SettingServerName, this.Nick, this.HostName, this.Channel, this.Topic);
        }
    }
}
