﻿using System;
using System.Globalization;
namespace Chocoat
{
    /// <summary>
    /// IRCのWhoイベントを受信した際の情報を提供します。
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [Serializable]
    public sealed class IRCWhoEventArgs : IRCEventArgs
    {
        /// <summary>
        /// IRCWhoEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="reValue">通知内容</param>
        public IRCWhoEventArgs(string settingServerName, string reValue)
            : base(settingServerName)
        {
            string[] args = reValue.Split(' ');

            this.Nick = args[0];
            this.HostName = args[1];
            this.Message = args[2];
        }

        /// <summary>
        /// ニックネームを取得します。
        /// </summary>
        public string Nick { get; private set; }

        /// <summary>
        /// ホスト名を取得します。
        /// </summary>
        public string HostName { get; private set; }

        /// <summary>
        /// メッセージを取得します。
        /// </summary>
        public string Message { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, Nick:{1}, HostName:{2}, Message:{3}",
                this.SettingServerName, this.Nick, this.HostName, this.Message);
        }
    }
}
