﻿using System;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// メッセージイベントを受信した際の情報を提供します。
    /// </summary>
    [Serializable]
    public sealed class MessageEventArgs : EventArgs
    {
        /// <summary>
        /// MessageEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="channel">チャンネル</param>
        /// <param name="nick">ニックネーム</param>
        /// <param name="address">マシンアドレス</param>
        /// <param name="messageType">メッセージの種類</param>
        /// <param name="message">メッセージ</param>
        internal MessageEventArgs(
            string settingServerName,
            string channel,
            string nick,
            string address,
            MessageType messageType,
            string message)
        {
            this.SettingServerName = settingServerName;
            this.Channel = channel;
            this.Nick = nick;
            this.Address = address;
            this.MessageType = messageType;
            this.Message = message;
        }

        /// <summary>
        /// サーバー設定名を取得します。
        /// </summary>
        public string SettingServerName { get; private set; }

        /// <summary>
        /// チャンネル名を取得します。
        /// </summary>
        public string Channel { get; private set; }

        /// <summary>
        /// ニックネームを取得します。
        /// </summary>
        public string Nick { get; private set; }

        /// <summary>
        /// マシンアドレスを取得します。
        /// </summary>
        public string Address { get; private set; }

        /// <summary>
        /// メッセージタイプを取得します。
        /// </summary>
        public MessageType MessageType { get; private set; }

        /// <summary>
        /// メッセージを取得します。
        /// </summary>
        public string Message { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, Channel:{1}, Nick:{2}, Address:{3}, MessageType:{4}, Message:{5}",
                this.SettingServerName, this.Channel, this.Nick, this.Address, this.MessageType, this.Message);
        }
    }
}
