﻿using System;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// Numericメッセージイベントを受信した際の情報を提供します。
    /// </summary>
    [Serializable]
    public class NumericMessageEventArgs : EventArgs
    {
        /// <summary>
        /// NumericMessageEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="numeric">ニュメリック値</param>
        /// <param name="message">メッセージ</param>
        internal NumericMessageEventArgs(
            string settingServerName,
            int numeric,
            string message)
        {
            this.SettingServerName = settingServerName;
            this.Numeric = numeric;
            this.Message = message;
        }

        /// <summary>
        /// サーバー設定名を取得します。
        /// </summary>
        public string SettingServerName { get; private set; }

        /// <summary>
        /// ニューメリックリプライ値を取得します。
        /// </summary>
        public int Numeric { get; private set; }

        /// <summary>
        /// 通知メッセージを取得します。
        /// </summary>
        public string Message { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, Numeric:{1}, Message:{2}",
                this.SettingServerName, this.Numeric, this.Message);
        }
    }
}
