﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;

namespace Chocoat
{
    /// <summary>
    /// Scriptイベントを受信した際の情報を提供します。
    /// </summary>
    [Serializable]
    public class ScriptEventArgs : EventArgs
    {
        /// <summary>
        /// ScriptEventArgs の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="settingServerName">サーバー設定名</param>
        /// <param name="eventType">イベントタイプ</param>
        /// <param name="serviceName">サービス名</param>
        /// <param name="functionName">コマンド名</param>
        /// <param name="returnValue">実行結果</param>
        internal ScriptEventArgs(
            string settingServerName,
            int eventType,
            string serviceName,
            string functionName,
            string returnValue)
        {
            this.SettingServerName = settingServerName;
            this.EventType = eventType;
            this.ServiceName = serviceName;
            this.FunctionName = functionName;
            this.ReturnValue = returnValue;
        }

        /// <summary>
        /// サーバー設定名を取得します。
        /// </summary>
        public string SettingServerName { get; private set; }

        /// <summary>
        /// 通知イベントを取得します。
        /// </summary>
        public int EventType { get; private set; }

        /// <summary>
        /// サービス名を取得します。
        /// </summary>
        public string ServiceName { get; private set; }

        /// <summary>
        /// コマンド名を取得します。
        /// </summary>
        public string FunctionName { get; private set; }

        /// <summary>
        /// 実行結果を取得します。
        /// </summary>
        public string ReturnValue { get; private set; }

        /// <summary>
        /// 現在の Object を表す String を返します。
        /// </summary>
        /// <returns>現在の Object を表す String。</returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.InstalledUICulture,
                "SettingServerName:{0}, EventType:{1}, ServiceName:{2}, FunctionName:{3}, ReturnValue:{4}",
                this.SettingServerName, this.EventType, this.ServiceName, this.FunctionName, this.ReturnValue);
        }
    }
}
