﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Globalization;

namespace Chocoat.Utility
{
    /// <summary>
    /// CHOCOA専用のIPアドレスの変換クラス
    /// </summary>
    public static class ChocoaIPConverter
    {
        /// <summary>
        /// IPアドレスをCHOCOA用のInt32の形に変換します。
        /// </summary>
        /// <param name="ipAddress">IPアドレス</param>
        /// <returns>取得した、CHOCOA用の Int32のIPAddress。</returns>
        public static int ToInt32(IPAddress ipAddress)
        {
            string[] split = ipAddress.ToString().Split('.');

            if (split.Length != 4)
            {
                return 0;
            }
            else
            {
                return ToInt32(
                    Convert.ToInt32(split[0], CultureInfo.InvariantCulture),
                    Convert.ToInt32(split[1], CultureInfo.InvariantCulture),
                    Convert.ToInt32(split[2], CultureInfo.InvariantCulture),
                    Convert.ToInt32(split[3], CultureInfo.InvariantCulture));
            }
        }

        /// <summary>
        /// IPアドレスの４つから、CHOCOA用のInt32を生成する
        /// </summary>
        /// <param name="b0">１</param>
        /// <param name="b1">２</param>
        /// <param name="b2">３</param>
        /// <param name="b3">４</param>
        /// <returns>CHOCOA用のInt32表現</returns>
        private static int ToInt32(int b0, int b1, int b2, int b3)
        {
            int result = b0 << 24;

            result += b1 << 16;
            result += b2 << 8;
            result += b3;

            return result;
        }

        /// <summary>
        /// CHOCOA用のInt32表現のIPアドレスを、IPAddressに変換します。
        /// </summary>
        /// <param name="chochoaIPAddress">CHOCOA用のInt32のIPアドレス</param>
        /// <returns>取得したIPアドレス。変換出来なかった時は <c>null</c> 。</returns>
        public static IPAddress ToIPAddress(int chochoaIPAddress)
        {
            int b0 = (chochoaIPAddress >> 24) & 0xff;
            int b1 = (chochoaIPAddress >> 16) & 0xff;
            int b2 = (chochoaIPAddress >> 8) & 0xff;
            int b3 = chochoaIPAddress & 0xff;

            string ipString = String.Format(CultureInfo.InvariantCulture, "{0}.{1}.{2}.{3}", b0, b1, b2, b3);

            IPAddress result = null;

            if (IPAddress.TryParse(ipString, out result))
            {
                return result;
            }
            else
            {
                return null;
            }
        }
    }
}
