﻿using System;

namespace Chocoat
{
    //このファイルはイベントハンドラをずらずらと書くもの

    /// <summary>
    /// メッセージイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void MessageEventHandler(object sender, MessageEventArgs e);

    /// <summary>
    /// スクリプトイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void ScriptEventHandler(object sender, ScriptEventArgs e);

    /// <summary>
    /// IRCのNickイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCNickEventHandler(object sender, IRCNickEventArgs e);

    /// <summary>
    /// IRCのJoinイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCJoinEventHandler(object sender, IRCJoinEventArgs e);

    /// <summary>
    /// IRCのTopicイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCTopicEventHandler(object sender, IRCTopicEventArgs e);

    /// <summary>
    /// IRCのNamesイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCNamesEventHandler(object sender, IRCNamesEventArgs e);

    /// <summary>
    /// IRCのPartイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCPartEventHandler(object sender, IRCPartEventArgs e);

    /// <summary>
    /// IRCのQuitイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCQuitEventHandler(object sender, IRCQuitEventArgs e);

    /// <summary>
    /// IRCのModeイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCModeEventHandler(object sender, IRCModeEventArgs e);

    /// <summary>
    /// IRCのKickイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCKickEventHandler(object sender, IRCKickEventArgs e);

    /// <summary>
    /// IRCのInviteイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCInviteEventHandler(object sender, IRCInviteEventArgs e);

    /// <summary>
    /// IRCのActionイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCActionEventHandler(object sender, IRCActionEventArgs e);

    /// <summary>
    /// IRCのIsonイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCIsonEventHandler(object sender, IRCIsonEventArgs e);

    /// <summary>
    /// IRCのWhoisイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCWhoisEventHandler(object sender, IRCWhoisEventArgs e);

    /// <summary>
    /// IRCのWhoイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "IRC")]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void IRCWhoEventHandler(object sender, IRCWhoEventArgs e);

    /// <summary>
    /// ニュメリックイベントを受信した時に発生するイベントハンドラの定義です。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void NumericMessageEventHandler(object sender, NumericMessageEventArgs e);

    /// <summary>
    /// CHOCOAのウィンドウの状態が変更された時に呼び出されるイベントです。
    /// </summary>
    /// <param name="sender">受信したオブジェクト</param>
    /// <param name="e">受信したデータの内容</param>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1003:UseGenericEventHandlerInstances")]
    public delegate void ChocoaWindowEventHandler(object sender, ChocoaWindowEventArgs e);
}
