﻿using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// ダイスの追加コントロール
    /// </summary>
    [DefaultEvent("ValueChanged")]
    public partial class AddDiceControl : UserControl
    {
        #region constructor
        /// <summary>
        /// AddDiceControl の新しいインスタンスを生成します。
        /// </summary>
        public AddDiceControl()
        {
            InitializeComponent();
        }
        #endregion

        #region chkUse_CheckedChanged / 利用のチェックが変更された
        /// <summary>
        /// 利用のチェックが変更された
        /// </summary>
        private void chkUse_CheckedChanged(object sender, EventArgs e)
        {
            foreach (NumericUpDown num in new NumericUpDown[] {
                this.numDiceAdd,
                this.numDiceSub,
                this.numBonusAdd,
                this.numBonusSub,
            })
            {
                num.Enabled = this.chkUse.Checked;
                num.ReadOnly = !num.Enabled;
            }

            foreach (CheckBox chk in new CheckBox[] {
                this.chkCritPlus1,
                this.chkCritPlus2,
                this.chkCritPlus3,
                this.chkCritPlus4,
                this.chkCritMinus1,
                this.chkCritMinus2,
                this.chkCritMinus3,
                this.chkCritMinus4,
            })
            {
                chk.Enabled = this.chkUse.Checked;
            }
            
            this.txtMemo.Text = String.Empty;

            this.OnValueChanged(new EventArgs());
        } 
        #endregion

        #region num_ValueChanged / 数値が変更された
        /// <summary>
        /// 数値が変更された
        /// </summary>
        private void num_ValueChanged(object sender, EventArgs e)
        {
            this.OnValueChanged(new EventArgs());
        }
        #endregion

        #region chkCrit_CheckedChanged / クリティカルのチェックが変更された
        /// <summary>
        /// クリティカルのチェックが変更された
        /// </summary>
        private void chkCrit_CheckedChanged(object sender, EventArgs e)
        {
            this.OnValueChanged(new EventArgs());
        }
        #endregion

        #region event ValueChanged / OnValueChanged / チェックが変更された時、値が変更された時に呼び出されます。
        /// <summary>
        /// チェックが変更された時、値が変更された時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Description("チェックが変更された時、値が変更された時に呼び出されます。")]
        [Category("Behavior")]
        public event EventHandler ValueChanged;

        /// <summary>
        /// <see cref="ValueChanged"/> を呼び出します。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        protected virtual void OnValueChanged(EventArgs e)
        {
            if (this.ValueChanged != null)
            {
                this.ValueChanged(this, e);
            }
        } 
        #endregion

        #region Use / 利用するなら true 。
        /// <summary>
        /// 利用するなら <c>true</c> 。
        /// </summary>
        [Browsable(false)]
        public bool Use
        {
            get
            {
                return this.chkUse.Checked;
            }
        }
        #endregion

        #region AddDice / 追加するダイスの数を取得します。
        /// <summary>
        /// 追加するダイスの数を取得します。
        /// </summary>
        [Browsable(false)]
        public int AddDice
        {
            get
            {
                return (this.chkUse.Checked) ? Convert.ToInt32(this.numDiceAdd.Value - this.numDiceSub.Value) : 0;
            }
        }
        #endregion

        #region SubtractCritical / 減算するクリティカル値を取得します。加算になるときは、-になります。
        /// <summary>
        /// 減算するクリティカル値を取得します。加算になるときは、-になります。
        /// </summary>
        [Browsable(false)]
        public int SubtractCritical
        {
            get
            {
                int result = 0;

                if (this.chkUse.Checked)
                {
                    foreach (CheckBox chk in new CheckBox[] {
                        this.chkCritMinus1,
                        this.chkCritMinus2,
                        this.chkCritMinus3,
                        this.chkCritMinus4,
                    })
                    {
                        if (chk.Checked)
                        {
                            result++;
                        }
                    }

                    foreach (CheckBox chk in new CheckBox[] {
                        this.chkCritPlus1,
                        this.chkCritPlus2,
                        this.chkCritPlus3,
                        this.chkCritPlus4,
                    })
                    {
                        if (chk.Checked)
                        {
                            result--;
                        }
                    }
                }

                return result;
            }
        }
        #endregion

        #region AddBonus / 追加するボーナス値を取得します。
        /// <summary>
        /// 追加するボーナス値を取得します。
        /// </summary>
        [Browsable(false)]
        public int AddBonus
        {
            get
            {
                return (this.chkUse.Checked) ? Convert.ToInt32(this.numBonusAdd.Value - this.numBonusSub.Value) : 0;
            }
        }
        #endregion
    }
}
