﻿using System;
using System.ComponentModel;
using System.Data;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// エフェクト追加用のForm
    /// </summary>
    public partial class AddEffect : Form
    {
        /// <summary>
        /// 操作する行
        /// </summary>
        private CharacterDataSet.CharacterEffectsRow _targetRow;

        #region constructor
        /// <summary>
        /// AddEffect の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="row">操作する行</param>
        public AddEffect(CharacterDataSet.CharacterEffectsRow row)
        {
            InitializeComponent();

            this._targetRow = row;

            foreach (ComboBox typeCombo in new ComboBox[]{
                this.cmbDiceType,
                this.cmbBonusType,
                this.cmbCritType,
            })
            {
                typeCombo.DataSource = EnumStringAttribute.GetEnumValueLabelArray(typeof(EffectTypes));
                typeCombo.ValueMember = "Key";
                typeCombo.DisplayMember = "Value";
            }

            foreach (ComboBox signCombo in new ComboBox[] {
                this.cmbDiceSign,
                this.cmbBonusSign,
                this.cmbCritSign,
            })
            {
                signCombo.DataSource = EnumStringAttribute.GetEnumValueLabelArray(typeof(EffectSign));
                signCombo.ValueMember = "Key";
                signCombo.DisplayMember = "Value";
            }
        } 
        #endregion

        #region OnLoad
        /// <summary>
        /// ロードされたとき
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (this._targetRow.RowState != DataRowState.Detached)
            {
                this.txtEffectName.Text = this._targetRow.EffectName;
                this.txtSummary.Text = this._targetRow.Summary;
                this.txtTiming.Text = this._targetRow.Timing;
                this.txtSkill.Text = this._targetRow.Timing;
                this.txtChallengeLevel.Text = this._targetRow.ChallengeLevel;
                this.txtTarget.Text = this._targetRow.Target;
                this.txtRange.Text = this._targetRow.Range;

                this.txtErosionRate.Text = this._targetRow.ErosionRate.ToString();
                this.txtErosionRateLimit.Text = this._targetRow.ErosionRateLimit.ToString();
                this.txtEffectLevel.Text = this._targetRow.EffectLevel.ToString();

                this.txtDiceBase.Text = this._targetRow.DiceBase.ToString();
                this.cmbDiceType.SelectedValue = this._targetRow.DiceType;
                this.txtDiceScale.Text = this._targetRow.DiceScale.ToString();
                this.txtDiceAdd.Text = this._targetRow.DiceAdd.ToString();
                this.cmbDiceSign.SelectedValue = this._targetRow.DiceSign;

                this.txtBonusBase.Text = this._targetRow.BonusBase.ToString();
                this.cmbBonusType.SelectedValue = this._targetRow.BonusType;
                this.txtBonusScale.Text = this._targetRow.BonusScale.ToString();
                this.txtBonusAdd.Text = this._targetRow.BonusAdd.ToString();
                this.cmbBonusSign.SelectedValue = this._targetRow.BonusSign;

                this.txtCritBase.Text = this._targetRow.CritBase.ToString();
                this.cmbCritType.SelectedValue = this._targetRow.CritType;
                this.txtCritScale.Text = this._targetRow.CritScale.ToString();
                this.txtCritAdd.Text = this._targetRow.CritAdd.ToString();
                this.cmbCritSign.SelectedValue = this._targetRow.CritSign;

                this.txtCritUnderLimit.Text = this._targetRow.CritUnderLimit.ToString();
            }
        }
        #endregion

        #region OnClosing / 閉じられるとき

        private bool _isClose = false;

        /// <summary>
        /// 閉じられるとき
        /// </summary>
        protected override void OnClosing(CancelEventArgs e)
        {
            if (!this._isClose)
            {
                e.Cancel = true;
            }

            base.OnClosing(e);
        }
        #endregion

        #region btnOK_Click / OKボタンクリック
        /// <summary>
        /// OKボタンクリック
        /// </summary>
        private void btnOK_Click(object sender, EventArgs e)
        {
            if (this.txtEffectName.TextLength == 0)
            {
                MessageBox.Show("エフェクト名を入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtErosionRate.Text, @"^[0-9\\-]+$"))
            {
                MessageBox.Show("侵蝕値は数値を入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtErosionRateLimit.Text, @"^[0-9]+$"))
            {
                MessageBox.Show("侵蝕値制限は数値を入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtEffectLevel.Text, @"^[0-9]+$"))
            {
                MessageBox.Show("エフェクトレベルは数値を入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtDiceBase.Text, @"^[0-9]+$") ||
                !Regex.IsMatch(this.txtDiceScale.Text, @"^[0-9]+$") ||
                !Regex.IsMatch(this.txtDiceAdd.Text, @"^[0-9]+$"))
            {
                MessageBox.Show("ダイス値は数値を入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtBonusBase.Text, @"^[0-9]+$") ||
                !Regex.IsMatch(this.txtBonusScale.Text, @"^[0-9]+$") ||
                !Regex.IsMatch(this.txtBonusAdd.Text, @"^[0-9]+$"))
            {
                MessageBox.Show("ボーナス値は数値を入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtCritBase.Text, @"^[0-9]+$") ||
                !Regex.IsMatch(this.txtCritScale.Text, @"^[0-9]+$") ||
                !Regex.IsMatch(this.txtCritAdd.Text, @"^[0-9]+$"))
            {
                MessageBox.Show("クリティカル値は数値を入力してください。");
                return;
            }
            else
            {
                this._targetRow.EffectName = this.txtEffectName.Text;
                this._targetRow.Summary = this.txtSummary.Text;
                this._targetRow.Timing = this.txtTiming.Text;
                this._targetRow.Skill = this.txtSkill.Text;
                this._targetRow.ChallengeLevel = this.txtChallengeLevel.Text;
                this._targetRow.Target = this.txtTarget.Text;
                this._targetRow.Range = this.txtRange.Text;

                this._targetRow.ErosionRate = Convert.ToInt32(this.txtErosionRate.Text);
                this._targetRow.ErosionRateLimit = Convert.ToInt32(this.txtErosionRateLimit.Text);
                this._targetRow.EffectLevel = Convert.ToInt32(this.txtEffectLevel.Text);

                this._targetRow.DiceBase = Convert.ToInt32(this.txtDiceBase.Text);
                this._targetRow.DiceType = Convert.ToInt32(this.cmbDiceType.SelectedValue);
                this._targetRow.DiceScale = Convert.ToInt32(this.txtDiceScale.Text);
                this._targetRow.DiceAdd = Convert.ToInt32(this.txtDiceAdd.Text);
                this._targetRow.DiceSign = Convert.ToInt32(this.cmbDiceSign.SelectedValue);

                this._targetRow.BonusBase = Convert.ToInt32(this.txtBonusBase.Text);
                this._targetRow.BonusType = Convert.ToInt32(this.cmbBonusType.SelectedValue);
                this._targetRow.BonusScale = Convert.ToInt32(this.txtBonusScale.Text);
                this._targetRow.BonusAdd = Convert.ToInt32(this.txtBonusAdd.Text);
                this._targetRow.BonusSign = Convert.ToInt32(this.cmbBonusSign.SelectedValue);

                this._targetRow.CritBase = Convert.ToInt32(this.txtCritBase.Text);
                this._targetRow.CritType = Convert.ToInt32(this.cmbCritType.SelectedValue);
                this._targetRow.CritScale = Convert.ToInt32(this.txtCritScale.Text);
                this._targetRow.CritAdd = Convert.ToInt32(this.txtCritAdd.Text);
                this._targetRow.CritSign = Convert.ToInt32(this.cmbCritSign.SelectedValue);

                this._targetRow.CritUnderLimit = Convert.ToInt32(this.txtCritUnderLimit.Text);

                this.DialogResult = DialogResult.OK;
                this._isClose = true;
                this.Close();
            }
        } 
        #endregion

        #region btnCancel_Click / 閉じる
        /// <summary>
        /// 閉じる
        /// </summary>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this._isClose = true;
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }
        #endregion
    }
}
