﻿using System;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// スキル追加ウィンドウ
    /// </summary>
    public partial class AddSkill : Form
    {
        #region constructor
        /// <summary>
        /// AddSkill の新しいインスタンスを生成します。
        /// </summary>
        public AddSkill()
        {
            InitializeComponent();
        }
        #endregion

        #region OnClosing / 閉じるとき

        private bool _isClose = false;

        /// <summary>
        /// 閉じるとき
        /// </summary>
        protected override void OnClosing(CancelEventArgs e)
        {
            if (!this._isClose)
            {
                e.Cancel = true;
            }

            base.OnClosing(e);
        }
        #endregion

        #region btnOK_Click / OKぼたん
        /// <summary>
        /// OKぼたん
        /// </summary>
        private void btnOK_Click(object sender, EventArgs e)
        {
            if (this.txtSkillName.TextLength == 0)
            {
                MessageBox.Show("スキル名を入力してください。");
                return;
            }
            else if (this.txtSkillLv.TextLength == 0)
            {
                MessageBox.Show("スキルレベルを入力してください。");
                return;
            }
            else if (!Regex.IsMatch(this.txtSkillLv.Text, @"^[0-9]+$"))
            {
                MessageBox.Show("スキルレベルは数値を入力してください。");
                return;
            }
            else
            {
                this.DialogResult = DialogResult.OK;
                this._isClose = true;
                this.Close();
            }
        }
        #endregion

        #region btnCancel_Click / 閉じる
        /// <summary>
        /// 閉じる
        /// </summary>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this._isClose = true;
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }
        #endregion

        //prop
        #region SkillName / スキル名を取得します。
        /// <summary>
        /// スキル名を取得します。
        /// </summary>
        public string SkillName
        {
            get
            {
                return this.txtSkillName.Text;
            }
        }
        #endregion

        #region SkillLv / スキルレベルを取得します。
        /// <summary>
        /// スキルレベルを取得します。
        /// </summary>
        public int SkillLv
        {
            get
            {
                return Convert.ToInt32(this.txtSkillLv.Text);
            }
        }
        #endregion
    }
}
