﻿using System;
using System.Data;
using System.Data.SqlServerCe;

namespace DoubleX3rd
{
    /// <summary>
    /// キャラクタのデータ用のデータセット
    /// </summary>
    public partial class CharacterDataSet
    {
        partial class CharacterEffectsDataTable
        {
        }

        #region GetCharacterDataMainDataTable / キャラクターのメインデータのテーブルを全件取得します。
        /// <summary>
        /// キャラクターのメインデータのテーブルを全件取得します。
        /// </summary>
        /// <returns>取得したテーブル</returns>
        public static CharacterDataSet.CharacterDataMainDataTable GetCharacterDataMainDataTable()
        {
            CharacterDataSet.CharacterDataMainDataTable table = new CharacterDataMainDataTable();

            using (CharacterDataSetTableAdapters.CharacterDataMainTableAdapter ta = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterDataMainTableAdapter())
            {
                ta.Fill(table);
            }

            return table;
        } 
        #endregion

        #region GetCharacterDataSet / 指定されたIDのキャラクターデータを取得します。
        /// <summary>
        /// 指定されたIDのキャラクターデータを取得します。
        /// </summary>
        /// <param name="pkid">ID</param>
        /// <returns>取得したデータ</returns>
        public static CharacterDataSet GetCharacterDataSet(string pkid)
        {
            CharacterDataSet result = new CharacterDataSet();

            using (CharacterDataSetTableAdapters.CharacterDataMainTableAdapter main = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterDataMainTableAdapter())
            using (CharacterDataSetTableAdapters.CharacterDataSkillTableAdapter skill = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterDataSkillTableAdapter())
            using (CharacterDataSetTableAdapters.CharacterEffectsTableAdapter effect = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterEffectsTableAdapter())
            {
                main.FillByPKID(result.CharacterDataMain, pkid);
                skill.FillByPKID(result.CharacterDataSkill, pkid);
                effect.FillByPKID(result.CharacterEffects, pkid);
            }

            return result;
        } 
        #endregion

        #region HasCharacterName / 指定されたキャラクター名のデータが存在するかを取得します。
        /// <summary>
        /// 指定されたキャラクター名のデータが存在するかを取得します。
        /// </summary>
        /// <param name="characterName">キャラクター名</param>
        /// <returns>存在すれば <c>true</c> 。</returns>
        public static bool HasCharacterName(string characterName)
        {
            SqlCeConnection conn = new SqlCeConnection(Properties.Settings.Default.ConnectionString);
            SqlCeCommand cmd = conn.CreateCommand();
            cmd.CommandText = @"SELECT COUNT(*) FROM [CharacterDataMain] WHERE [CharacterName] = @CharacterName";
            cmd.Parameters.Add("@CharacterName", SqlDbType.NVarChar).Value = characterName;

            using (conn)
            using (cmd)
            {
                conn.Open();
                object obj = cmd.ExecuteScalar();

                return (obj != null && Convert.ToDecimal(obj) > 0);
            }
        } 
        #endregion

        #region Update / 更新します。
        /// <summary>
        /// 更新します。
        /// </summary>
        /// <returns>更新した件数</returns>
        public int Update()
        {
            //ここ、ちょっと適当にやりすぎたので、ちょっと適当に処理しておく
            int result = 0;

            using (CharacterDataSetTableAdapters.TableAdapterManager ta = new DoubleX3rd.CharacterDataSetTableAdapters.TableAdapterManager())
            using (ta.CharacterDataMainTableAdapter = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterDataMainTableAdapter())
            using (ta.CharacterDataSkillTableAdapter = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterDataSkillTableAdapter())
            using (ta.CharacterEffectsTableAdapter = new DoubleX3rd.CharacterDataSetTableAdapters.CharacterEffectsTableAdapter())
            {
                SqlCeConnection conn = (SqlCeConnection)ta.Connection;
                conn.Open();

                using (conn)
                using (SqlCeTransaction tran = conn.BeginTransaction())
                {
                    ta.CharacterDataMainTableAdapter.Connection = conn;
                    ta.CharacterDataMainTableAdapter.Transaction = tran;

                    ta.CharacterDataSkillTableAdapter.Connection = conn;
                    ta.CharacterDataSkillTableAdapter.Transaction = tran;

                    ta.CharacterEffectsTableAdapter.Connection = conn;
                    ta.CharacterEffectsTableAdapter.Transaction = tran;

                    string pkid = (this.CharacterDataMain[0].RowState != DataRowState.Deleted) ? this.CharacterDataMain[0].PKID : (string)this.CharacterDataMain[0][this.CharacterDataMain.PKIDColumn, DataRowVersion.Original];

                    foreach (string tableName in new string[] { "CharacterDataMain", "CharacterDataSkill", "CharacterEffects" })
                    {
                        SqlCeCommand cmd = conn.CreateCommand();
                        cmd.Transaction = tran;
                        cmd.CommandText = String.Format("DELETE FROM [{0}] WHERE ([PKID] = @p1)", tableName);
                        cmd.Parameters.Add("@p1", SqlDbType.NChar, 36).Value = pkid;

                        cmd.ExecuteNonQuery();
                    }

                    this.AcceptChanges();

                    foreach (DataTable tbl in this.Tables)
                    {
                        foreach (DataRow row in tbl.Rows)
                        {
                            row.SetAdded();
                        }
                    }

                    ta.CharacterDataMainTableAdapter.Update(this);
                    ta.CharacterDataSkillTableAdapter.Update(this);
                    ta.CharacterEffectsTableAdapter.Update(this);

                    tran.Commit();
                }
            }
        
            this.AcceptChanges();

            return result;
        } 
        #endregion
    }
}
