﻿using System;

namespace DoubleX3rd
{
    /// <summary>
    /// ダイスを扱うための構造体
    /// </summary>
    public struct DiceRoll
    {
        /// <summary>
        /// 「クリティカルしない」に利用する定数
        /// </summary>
        public const int NotCritical = 99;

        /// <summary>
        /// クリティカル値の最低値の定数
        /// </summary>
        public const int MinCritical = 2;

        #region Dice / ダイスの個数を取得、または設定します。

        private int _dice;

        /// <summary>
        /// ダイスの個数を取得、または設定します。
        /// </summary>
        public int Dice
        {
            get
            {
                return this._dice;
            }
            set
            {
                this._dice = value;
            }
        } 
        #endregion
        
        #region Critical / クリティカル値を取得、または設定します。

        private int _critical;

        /// <summary>
        /// クリティカル値を取得、または設定します。
        /// </summary>
        public int Critical
        {
            get
            {
                if (this._critical == 0)
                {
                    return NotCritical;
                }

                return this._critical;
            }
            set
            {
                if (MinCritical > value)
                {
                    this._critical = MinCritical;
                }
                else
                {
                    this._critical = value;
                }
            }
        } 
        #endregion

        #region Bonus / ボーナス値を取得、または設定します。
        /// <summary>
        /// ボーナス値を取得、または設定します。
        /// </summary>
        public int Bonus { get; set; } 
        #endregion

        #region ToString / 文字列にする
        /// <summary>
        /// 文字列にする
        /// </summary>
        /// <returns>文字列</returns>
        public override string ToString()
        {
            if (this.Critical == NotCritical)
            {
                return String.Format("{0}D10 {1} {2}", this.Dice, (0 > this.Bonus) ? "-" : "+", Math.Abs(this.Bonus));
            }
            else
            {
                return String.Format("{0}D10 {1} {2} / cri:{3}", this.Dice, (0 > this.Bonus) ? "-" : "+", Math.Abs(this.Bonus), this.Critical);
            }
        } 
        #endregion

        //static
        #region NewDiceRole / ダイスを生成する
        /// <summary>
        /// ダイスを生成する
        /// </summary>
        /// <param name="dice">ダイスの数</param>
        /// <param name="bonus">ボーナス</param>
        /// <param name="crit">クリティカル値</param>
        /// <param name="underCrit">クリティカル下限値</param>
        /// <returns>生成されたダイス</returns>
        public static DiceRoll NewDiceRole(int dice, int bonus, int crit)
        {
            return NewDiceRole(dice, bonus, crit, 2);
        }

        /// <summary>
        /// ダイスを生成する
        /// </summary>
        /// <param name="dice">ダイスの数</param>
        /// <param name="bonus">ボーナス</param>
        /// <param name="crit">クリティカル値</param>
        /// <param name="underCrit">クリティカル下限値</param>
        /// <returns>生成されたダイス</returns>
        public static DiceRoll NewDiceRole(int dice, int bonus, int crit, int underCrit)
        {
            DiceRoll result = new DiceRoll();

            result.Dice = dice;
            result.Bonus = bonus;
            result.Critical = crit;

            if (underCrit > result.Critical)
            {
                result.Critical = underCrit;
            }

            return result;
        } 
        #endregion
    }
}
