﻿using System;
using System.ComponentModel;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// 侵蝕値の管理コントロール
    /// </summary>
    [DefaultEvent("ErosionRateChanged")]
    public partial class ErosionRateControl : UserControl
    {
        #region constructor
        /// <summary>
        /// ErosionRateControl の新しいインスタンスを生成します。
        /// </summary>
        public ErosionRateControl()
        {
            InitializeComponent();
        } 
        #endregion

        #region chkBeginning_CheckedChanged / 起源種のチェックを変更したら、ボーナスの再計算
        /// <summary>
        /// 起源種のチェックを変更したら、ボーナスの再計算
        /// </summary>
        private void chkBeginning_CheckedChanged(object sender, EventArgs e)
        {
            this.DiceBonus = GetDiceBonus(this.ErosionRate, this.chkBeginning.Checked);
            this.txtAddD10.Text = this.DiceBonus.ToString();

            this.EffectLevelBonus = GetEffectLevelBonus(this.ErosionRate, this.chkBeginning.Checked);
            this.txtAddEffectD10.Text = this.EffectLevelBonus.ToString();

            this.OnErosionRateChanged(new ErosionRateChangedEventArgs(this.ErosionRate));
        } 
        #endregion

        #region txtAddText_KeyUp / ここに入力して、Enterで使う。マイナス入るんで、入れすぎたら下げろ
        /// <summary>
        /// ここに入力して、Enterで使う。マイナス入るんで、入れすぎたら下げろ
        /// </summary>
        private void txtAddText_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                btnChangeNick.PerformClick();
                e.Handled = true;
            }
        } 
        #endregion

        #region btnChangeNick_Click / 変更ぼたん
        /// <summary>
        /// 変更ぼたん
        /// </summary>
        private void btnChangeNick_Click(object sender, EventArgs e)
        {
            if (this.txtAddText.TextLength > 0)
            {
                if (Regex.IsMatch(this.txtAddText.Text, @"^\-?[0-9]+$"))
                {
                    int addValue = Convert.ToInt32(this.txtAddText.Text);

                    if (this.StartErosionRate == 0)
                    {
                        this.StartErosionRate = addValue;
                    }
                    
                    StringBuilder itemString = new StringBuilder(String.Format("{0} + {1} = {2}", this.ErosionRate, addValue, this.ErosionRate += addValue));

                    if (this.txtMemo.TextLength > 0)
                    {
                        itemString.AppendFormat("({0})", this.txtMemo.Text);
                    }

                    string newItem = itemString.ToString();

                    this.lstHistory.TopIndex = this.lstHistory.Items.Add(newItem);

                    if (0 > this.ErosionRate)
                    {
                        this.ErosionRate = 0;
                        this.lstHistory.TabIndex = this.lstHistory.Items.Add("0");
                    }

                    //使いやすい
                    this.txtAddText.Text = "0";
                    this.txtAddText.SelectAll();

                    this.DiceBonus = GetDiceBonus(this.ErosionRate, this.chkBeginning.Checked);
                    this.txtAddD10.Text = this.DiceBonus.ToString();

                    this.EffectLevelBonus = GetEffectLevelBonus(this.ErosionRate, this.chkBeginning.Checked);
                    this.txtAddEffectD10.Text = this.EffectLevelBonus.ToString();
                    this.txtMemo.Text = String.Empty;

                    this.OnErosionRateChanged(new ErosionRateChangedEventArgs(this.ErosionRate));
                }
            }
        } 
        #endregion

        #region GetDiceBonus / 侵蝕値から、ダイスボーナスを取得する
        /// <summary>
        /// 侵蝕値から、ダイスボーナスを取得する
        /// </summary>
        /// <param name="erosionRate">侵蝕値</param>
        /// <param name="isBeginning">起源種（上級ルール）であるか</param>
        /// <returns>取得したダイスボーナス（ダイス個）</returns>
        public static int GetDiceBonus(int erosionRate, bool isBeginning)
        {
            //起源種には、ダイス追加がない
            if (!isBeginning)
            {
                if (erosionRate >= 300)
                {
                    return 8;
                }
                else if (erosionRate >= 240)
                {
                    return 7;
                }
                else if (erosionRate >= 200)
                {
                    return 6;
                }
                else if (erosionRate >= 160)
                {
                    return 5;
                }
                else if (erosionRate >= 130)
                {
                    return 4;
                }
                else if (erosionRate >= 100)
                {
                    return 3;
                }
                else if (erosionRate >= 80)
                {
                    return 2;
                }
                else if (erosionRate >= 60)
                {
                    return 1;
                }
            }

            return 0;
        }
        #endregion

        #region GetEffectLevelBonus / 侵蝕値から、エフェクトボーナスを取得します。
        /// <summary>
        /// 侵蝕値から、エフェクトボーナスを取得します。
        /// </summary>
        /// <param name="erosionRate">侵蝕値</param>
        /// <param name="isBeginning">起源種（上級ルール）であるか</param>
        /// <returns>取得したエフェクトボーナス（ダイス個）</returns>
        public static int GetEffectLevelBonus(int erosionRate, bool isBeginning)
        {
            if (isBeginning)
            {
                if (erosionRate >= 150)
                {
                    return 3;
                }
                else if (erosionRate >= 100)
                {
                    return 2;
                }
                else if (erosionRate >= 80)
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
            else
            {
                if (erosionRate >= 160)
                {
                    return 2;
                }
                else if (erosionRate >= 100)
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
        }
        #endregion

        #region ErosionRateChanged / 侵蝕値が変更された時に呼び出されます。
        /// <summary>
        /// 侵蝕値が変更された時に呼び出されます。
        /// </summary>
        [Browsable(true)]
        [Description("侵蝕値が変更された時に呼び出されます。")]
        [Category("Behavior")]
        public event ErosionRateChangedEventHandler ErosionRateChanged;

        /// <summary>
        /// <see cref="ErosionRateChanged"/> イベントを発生させます。
        /// </summary>
        /// <param name="e">イベントデータ</param>
        protected virtual void OnErosionRateChanged(ErosionRateChangedEventArgs e)
        {
            if (this.ErosionRateChanged != null)
            {
                this.ErosionRateChanged(this, e);
            }
        }
        #endregion 

        #region btnClear_Click / クリア
        /// <summary>
        /// クリア
        /// </summary>
        private void btnClear_Click(object sender, EventArgs e)
        {
            this.lstHistory.Items.Clear();
            this.ErosionRate = 0;
            this.StartErosionRate = 0;
        }
        #endregion

        #region SetDefaultErosionRate / 侵蝕値を指定して初期化する
        /// <summary>
        /// 侵蝕値を指定して初期化する
        /// </summary>
        /// <param name="erosionRate">侵蝕値</param>
        public void SetDefaultErosionRate(int erosionRate, bool isBeginning)
        {
            //初期化して、値を設定
            this.btnClear.PerformClick();

            this.txtAddText.Text = erosionRate.ToString();
            this.chkBeginning.Checked = isBeginning;

            this.btnChangeNick.PerformClick();
        }
        #endregion

        #region AddErosionRate / 侵蝕値を加算します。
        /// <summary>
        /// 侵蝕値を加算します。
        /// </summary>
        /// <param name="addRate">加算する値</param>
        /// <returns>加算後の侵蝕値</returns>
        public int AddErosionRate(int addRate)
        {
            string temp = this.txtAddText.Text;

            this.txtAddText.Text = addRate.ToString();
            this.btnChangeNick.PerformClick();

            this.txtAddText.Text = temp;

            return this.ErosionRate;
        }  
        #endregion

        /// <summary>
        /// 初期値の侵蝕率
        /// </summary>
        [Browsable(false)]
        public int StartErosionRate { get; private set; }

        /// <summary>
        /// 侵蝕値を取得します。
        /// </summary>
        [Browsable(false)]
        public int ErosionRate { get; private set; }

        /// <summary>
        /// 起源種であるかを取得します。
        /// </summary>
        [Browsable(false)]
        public bool IsBeginning
        {
            get { return this.chkBeginning.Checked; }
        }
        /// <summary>
        /// ダイスボーナスを取得します。
        /// </summary>
        [Browsable(false)]
        public int DiceBonus { get; private set; }

        /// <summary>
        /// エフェクトレベルボーナスを取得します。
        /// </summary>
        [Browsable(false)]
        public int EffectLevelBonus { get; private set; }
    }
}
