﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// DX3用の画面のメインです。
    /// </summary>
    public partial class Form1 : Form
    {
        #region constructor
        /// <summary>
        /// Form1 の新しいインスタンスを生成します。
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            //クリティカル下限値
            List<KeyValuePair<int, string>> critUnder = new List<KeyValuePair<int,string>>();

            for (int i = 10; 2 <= i; i--)
            {
                critUnder.Add(new KeyValuePair<int, string>(i, i.ToString()));
            }

            critUnder.Add( new KeyValuePair<int,string>(DiceRoll.NotCritical, "しない"));

            this.cmbCritUnderLimit.ValueMember = "Key";
            this.cmbCritUnderLimit.DisplayMember = "Value";
            this.cmbCritUnderLimit.DataSource = critUnder;
           
            this.cmbCritUnderLimit.SelectedIndex = 0;
        }
        #endregion

        #region OnLoad
        /// <summary>
        /// ロードされたらコンボ作っておく
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            //接続しているサーバーのリスト
            this.RefreshServers();

            //メニューの生成
            this.CreateContextMenu();
        }
        #endregion

        #region OnClosing
        /// <summary>
        /// OnClosing
        /// </summary>
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            if (MessageBox.Show("終了しますか？", "確認", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
            {
                e.Cancel = true;
            }
            base.OnClosing(e);
        }
        #endregion

        private void ResetForm()
        {
            this.dsDataSet.Clear();

        }
    
        #region コンテキストメニュー周り (手抜き

        #region CreateContextMenu / メニューを作成する
        /// <summary>
        /// メニューを作成する
        /// </summary>
        private void CreateContextMenu()
        {
            this.ctMenu.Items.Clear();

            this.ctMenu.Items.Add("新規として保存する", Properties.Resources.NewDocument, this.SaveNew);

            CharacterDataSet.CharacterDataMainDataTable table = CharacterDataSet.GetCharacterDataMainDataTable();

            if (table.Count > 0)
            {
                this.ctMenu.Items.Add("-");

                foreach (CharacterDataSet.CharacterDataMainRow row in table)
                {
                    ToolStripItem item = this.ctMenu.Items.Add(row.CharacterName, Properties.Resources.OpenSelectedItem, this.LoadData);
                    item.Tag = row.PKID;
                }
            }
        }
        #endregion

        #region LoadData / データのロード（手抜き
        /// <summary>
        /// データのロード（手抜き
        /// </summary>
        private void LoadData(object sender, EventArgs e)
        {
            if (MessageBox.Show("データをロードします。よろしいですか？", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                ToolStripMenuItem item = (ToolStripMenuItem)sender;

                string pkid = (string)item.Tag;

                this.dsDataSet.Clear();
                this.dsDataSet.Merge(CharacterDataSet.GetCharacterDataSet(pkid));

                //メインのデータがあれば、そのデータを各コントロールにつける
                if (this.dsDataSet.CharacterDataMain.Count > 0)
                {
                    CharacterDataSet.CharacterDataMainRow mainRow = this.dsDataSet.CharacterDataMain[0];
                    this.txtNick.Text = mainRow.CharacterName;
                    this.erosionRateControl.SetDefaultErosionRate(mainRow.DefaultErosionRate, mainRow.IsBeginning);

                    Dictionary<int, Action<SkillListControl.SkillListItem[]>> dic = new Dictionary<int, Action<SkillListControl.SkillListItem[]>>(4);

                    dic.Add(1, delegate(SkillListControl.SkillListItem[] skills)
                    {
                        this.statusControl.StatusBody = mainRow.StatusBody;
                        this.statusControl.SkillBody = skills;
                    });

                    dic.Add(2, delegate(SkillListControl.SkillListItem[] skills)
                    {
                        this.statusControl.StatusSense = mainRow.StatusSense;
                        this.statusControl.SkillSense = skills;
                    });

                    dic.Add(3, delegate(SkillListControl.SkillListItem[] skills)
                    {
                        this.statusControl.StatusMental = mainRow.StatusMental;
                        this.statusControl.SkillMental = skills;
                    });

                    dic.Add(4, delegate(SkillListControl.SkillListItem[] skills)
                    {
                        this.statusControl.StatusSociety = mainRow.StatusSociety;
                        this.statusControl.SkillSociety = skills;
                    });

                    foreach (int statusKbn in dic.Keys)
                    {
                        DataView view = this.dsDataSet.CharacterDataSkill.DefaultView;
                        view.RowFilter = String.Format("StatusKbn = '{0}'", statusKbn);
                        view.Sort = "SerialNo";

                        List<SkillListControl.SkillListItem> skillList = new List<SkillListControl.SkillListItem>(view.Count);
                        foreach (DataRowView rowView in view)
                        {
                            CharacterDataSet.CharacterDataSkillRow row = (CharacterDataSet.CharacterDataSkillRow)rowView.Row;
                            skillList.Add(new SkillListControl.SkillListItem(row.SkillName, row.SkillLevel));
                        }

                        dic[statusKbn](skillList.ToArray());
                    }

                    this.effectControl.BindDataTable(this.dsDataSet.CharacterEffects);
                }

                //子要素をクリア
                foreach (ToolStripItem child in this.ctMenu.Items)
                {
                    ToolStripMenuItem childItem = child as ToolStripMenuItem;

                    if (childItem != null && childItem.HasDropDownItems)
                    {
                        childItem.DropDownItems.Clear();
                    }
                }

                ToolStripItem subItem = item.DropDownItems.Add("保存", Properties.Resources.Save, this.SaveData);
                subItem.Tag = pkid;

                subItem = item.DropDownItems.Add("削除", Properties.Resources.Delete, this.DeleteData);
                subItem.Tag = pkid;

                this.ctMenu.Close();

                this.cmbCritUnderLimit.SelectedIndex = 0;
            }
        }
        #endregion

        #region SaveNew / 新規保存
        /// <summary>
        /// 新規保存
        /// </summary>
        private void SaveNew(object sender, EventArgs e)
        {
            //現在の状態を回収する
            if (this.txtNick.TextLength == 0 || CharacterDataSet.HasCharacterName(this.txtNick.Text))
            {
                MessageBox.Show("NICKがないか、すでに登録されている名前は登録出来ません。");
            }
            else if (MessageBox.Show("保存しますか？", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                //手抜きなんでな
                this.dsDataSet.Clear();
                
                string pkid = Guid.NewGuid().ToString();

                this.LoadDataSet(pkid);

                if (this.dsDataSet.Update() > 0)
                {
                    //メニューを再生成して、このIDを選択
                    this.CreateContextMenu();

                    //どっこかなー
                    foreach (ToolStripItem item in this.ctMenu.Items)
                    {
                       if (item is ToolStripMenuItem && (string)item.Tag == pkid)
                       {
                           this.LoadData(item, new EventArgs());
                       }
                    }
                }
            }
        } 
        #endregion

        #region SaveData / 保存
        /// <summary>
        /// 保存
        /// </summary>
        private void SaveData(object sender, EventArgs e)
        {
            //現在の状態を回収して更新
            if (this.txtNick.TextLength == 0)
            {
                MessageBox.Show("NICKがない状態は、登録出来ません。");
            }
            else if (MessageBox.Show("保存しますか？", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                ToolStripMenuItem item = (ToolStripMenuItem)sender;

                string pkid = (string)item.Tag;

                //全部削除して、Loadで復元させる仕組み
                foreach (DataTable table in this.dsDataSet.Tables)
                {
                    for (int i = table.Rows.Count; 0 < i; i--)
                    {
                        table.Rows[i - 1].Delete();
                    }
                }

                this.LoadDataSet(pkid);

                this.dsDataSet.Update();
                this.effectControl.BindDataTable(this.dsDataSet.CharacterEffects);
            }
        } 
        #endregion

        #region LoadDataSet / 新規と保存の実態

        //3.5ならあるのに
        public delegate TResult Func<TResult>();

        /// <summary>
        /// 新規と保存の実態
        /// </summary>
        /// <param name="pkid">キー</param>
        private void LoadDataSet(string pkid)
        {
            //新規も更新も、内部的にはこれで処理する。めどいので
            CharacterDataSet.CharacterDataMainRow mainRow = this.dsDataSet.CharacterDataMain.NewCharacterDataMainRow();

            mainRow.PKID = pkid;
            mainRow.CharacterName = this.txtNick.Text;
            mainRow.DefaultErosionRate = this.erosionRateControl.StartErosionRate;
            mainRow.IsBeginning = this.erosionRateControl.IsBeginning;
            mainRow.StatusBody = this.statusControl.StatusBody;
            mainRow.StatusSense = this.statusControl.StatusSense;
            mainRow.StatusMental = this.statusControl.StatusMental;
            mainRow.StatusSociety = this.statusControl.StatusSociety;

            this.dsDataSet.CharacterDataMain.LoadDataRow(mainRow.ItemArray, false);

            Dictionary<string, Func<SkillListControl.SkillListItem[]>> dic = new Dictionary<string, Func<SkillListControl.SkillListItem[]>>();

            dic.Add("1", delegate() { return this.statusControl.SkillBody; });
            dic.Add("2", delegate() { return this.statusControl.SkillSense; });
            dic.Add("3", delegate() { return this.statusControl.SkillMental; });
            dic.Add("4", delegate() { return this.statusControl.SkillSociety; });

            foreach (string statusKbn in dic.Keys)
            {
                SkillListControl.SkillListItem[] skills = dic[statusKbn]();

                for (int i = 0; i < skills.Length; i++)
                {
                    SkillListControl.SkillListItem item = skills[i];

                    CharacterDataSet.CharacterDataSkillRow skillRow = this.dsDataSet.CharacterDataSkill.NewCharacterDataSkillRow();
                    skillRow.PKID = pkid;
                    skillRow.StatusKbn = statusKbn;
                    skillRow.SerialNo = i + 1;
                    skillRow.SkillName = item.SkillName;
                    skillRow.SkillLevel = item.SkillLevel;

                    this.dsDataSet.CharacterDataSkill.LoadDataRow(skillRow.ItemArray, false);
                }
            }

            //エフェクト部分の回収
            this.effectControl.UpdateDataTable(this.dsDataSet.CharacterEffects);

            List<int> removeRow = new List<int>();

            for (int i = 0; i < this.dsDataSet.CharacterEffects.Count; i++)
            {
                CharacterDataSet.CharacterEffectsRow row = this.dsDataSet.CharacterEffects[i];

                string rowPKID = row.HasVersion(DataRowVersion.Original) ? (string)row["PKID", DataRowVersion.Original] : null;

                if (rowPKID != pkid)
                {
                    //これは新規の行として作るはずなので
                    switch (row.RowState)
                    {
                        case DataRowState.Unchanged:
                        case DataRowState.Modified:
                            row.PKID = pkid;
                            row.AcceptChanges();
                            row.SetAdded();
                            break;
                        case DataRowState.Added:
                            row.PKID = pkid;
                            break;
                        case DataRowState.Deleted:
                            removeRow.Add(i);
                            break;
                    }
                }
            }

            for (int i = removeRow.Count; 0 < i; i--)
            {
                this.dsDataSet.CharacterEffects.Rows.RemoveAt(removeRow[i - 1]);
            }
        }
        #endregion

        #region DeleteData / 削除
        /// <summary>
        /// 削除
        /// </summary>
        private void DeleteData(object sender, EventArgs e)
        {
            if (MessageBox.Show("削除しますか？", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                //すべて削除
                foreach (DataTable table in this.dsDataSet.Tables)
                {
                    for (int i = table.Rows.Count; 0 < i; i--)
                    {
                        table.Rows[i - 1].Delete();
                    }
                }

                if (this.dsDataSet.Update() > 0)
                {
                    this.ResetForm();
                    this.CreateContextMenu();
                }
            }
        } 
        #endregion

        #endregion

        #region cmbConnectedServers_SelectedIndexChanged / サーバ変わったらリストとれよ
        /// <summary>
        /// サーバ変わったらリストとれよ
        /// </summary>
        private void cmbConnectedServers_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.cmbConnectedServers.SelectedIndex != -1)
            {
                string serverName = (string)this.cmbConnectedServers.SelectedItem;

                this.cmbChannels.Items.Clear();

                int serverID = this.chocoa.FindServerID(serverName);

                if (serverID != 0)
                {
                    foreach (string item in this.chocoa.GetChannelList(serverID))
                    {
                        this.cmbChannels.Items.Add(item);
                    }

                    if (this.cmbChannels.Items.Count > 0)
                    {
                        this.cmbChannels.SelectedIndex = 0;
                    }
                }
            }
        } 
        #endregion

        #region btnRefresh_Click / コンボの更新
        /// <summary>
        /// コンボの更新
        /// </summary>
        private void btnRefresh_Click(object sender, EventArgs e)
        {
            this.RefreshServers();
        }
        #endregion

        #region RefreshServers / コンボの更新の本体
        /// <summary>
        /// コンボの更新の本体
        /// </summary>
        private void RefreshServers()
        {
            this.cmbConnectedServers.Items.Clear();
            this.cmbChannels.Items.Clear();

            foreach (string item in this.chocoa.GetConnectedServerList())
            {
                this.cmbConnectedServers.Items.Add(item);
            }

            if (this.cmbConnectedServers.Items.Count > 0)
            {
                this.cmbConnectedServers.SelectedIndex = 0;
                string connectedServerName = (string)this.cmbConnectedServers.SelectedItem;

                int serverID = this.chocoa.FindServerID(connectedServerName);

                if (serverID != 0)
                {
                    foreach (string item in this.chocoa.GetChannelList(serverID))
                    {
                        this.cmbChannels.Items.Add(item);
                    }

                    if (this.cmbChannels.Items.Count > 0)
                    {
                        this.cmbChannels.SelectedIndex = 0;
                    }
                }
            }
        }
        #endregion

        #region erosionRateControl_ErosionRateChanged / 侵蝕値の更新
        /// <summary>
        /// 侵蝕値の更新
        /// </summary>
        private void erosionRateControl_ErosionRateChanged(object sender, ErosionRateChangedEventArgs e)
        {
            if (this.chkSendErosionRate.Checked && this.txtNick.TextLength > 0)
            {
                string serverName = (string)this.cmbConnectedServers.SelectedItem;
                int serverID = this.chocoa.FindServerID(serverName);
                string channel = (string)this.cmbChannels.SelectedItem;

                int maxLen = 15;

                if (this.rbNickLen9.Checked)
                {
                    maxLen = 9;
                }
                else if (this.rbNickLen15.Checked)
                {
                    maxLen = 15;
                }
                else if (this.rbNickLen30.Checked)
                {
                    maxLen = 30;
                }
                
                StringBuilder newNick = new StringBuilder(maxLen);

                newNick.Append(e.ErosionRate);

                for(int i=0;i<this.txtNick.TextLength;i++)
                {
                    newNick.Insert(i, this.txtNick.Text[i]);

                    if (newNick.Length >= maxLen)
                    {
                        break;
                    }
                }
                if (maxLen > newNick.Length)
                {
                    newNick.Insert(this.txtNick.TextLength, "_");
                }

                this.chocoa.ChangeNick(serverID, newNick.ToString());
            }

            this.CalcDiceRoll();
        } 
        #endregion

        #region statusControl_DiceRollChanged / ステータスの選択
        /// <summary>
        /// ステータスの選択
        /// </summary>
        private void statusControl_DiceRollChanged(object sender, DiceRollChangedEventArgs e)
        {
            this.CalcDiceRoll();
        } 
        #endregion

        #region addDiceControl_ValueChanged / 追加ダイスの更新
        /// <summary>
        /// 追加ダイスの更新
        /// </summary>
        private void addDiceControl_ValueChanged(object sender, EventArgs e)
        {
            this.CalcDiceRoll();
        }
        #endregion

        #region effectControl_CheckedChanged / エフェクト部分のチェック
        /// <summary>
        /// エフェクト部分のチェック
        /// </summary>
        private void effectControl_CheckedChanged(object sender, EventArgs e)
        {
            this.CalcDiceRoll();
        }
        #endregion

        #region CalcDiceRoll / ダイスの数を数えるところ
        /// <summary>
        /// ダイスの数を数えるところ
        /// </summary>
        private void CalcDiceRoll()
        {
            //基本のダイス
            DiceRoll dice = this.statusControl.SelectedDiceRoll;

            //侵蝕率ボーナス
            dice.Dice += this.erosionRateControl.DiceBonus;

            //修正
            foreach (AddDiceControl ctrl in new AddDiceControl[] {
                this.addDiceControl1,
                this.addDiceControl2,
                this.addDiceControl3,
                this.addDiceControl4,
            })
            {
                dice.Dice += ctrl.AddDice;
                dice.Critical -= ctrl.SubtractCritical;
                dice.Bonus += ctrl.AddBonus;
            }

            //エフェクト分
            dice.Dice += this.effectControl.Dice;
            dice.Bonus += this.effectControl.Bonus;

            if (dice.Critical == DiceRoll.NotCritical)
            {
                dice.Critical = 10;
            }

            dice.Critical -=this.effectControl.MinusCritical;

            this._lastCreateDice = dice;

            this.txtDiceRollPresent.Text = this._lastCreateDice.ToString();

            //下限値の調整
            if (this.cmbCritUnderLimit.SelectedIndex != -1)
            {
                if ((int)this.cmbCritUnderLimit.SelectedValue > this.effectControl.UnderCritical)
                {
                    this.cmbCritUnderLimit.SelectedValue = this.effectControl.UnderCritical;
                }
            }

            this.txtDiceRoll.Text = this.CreateAdjustedUnderLimitCriticalDice().ToString();

            this.btnCreateEffectText.PerformClick();
        }
        #endregion

        #region cmbCritUnderLimit_SelectedIndexChanged / クリティカル下限値の変更
        /// <summary>
        /// クリティカル下限値の変更
        /// </summary>
        private void cmbCritUnderLimit_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.cmbCritUnderLimit.SelectedIndex != -1)
            {
                //表示だけかえるのよ
                this.txtDiceRoll.Text = this.CreateAdjustedUnderLimitCriticalDice().ToString();
            }
        }
        #endregion

        #region CreateAdjustedUnderLimitCriticalDice / クリティカル下限値にあわせるダイスを生成する
        /// <summary>
        /// クリティカル下限値にあわせるダイスを生成する
        /// </summary>
        /// <returns>クリティカル下限値にあわされたダイス</returns>
        private DiceRoll CreateAdjustedUnderLimitCriticalDice()
        {
            DiceRoll result = this._lastCreateDice;

            int crit = this._lastCreateDice.Critical;

            if (this.cmbCritUnderLimit.SelectedIndex != -1)
            {
                if ((int)this.cmbCritUnderLimit.SelectedValue == DiceRoll.NotCritical)
                {
                    crit = DiceRoll.NotCritical;
                }
                else if ((int)this.cmbCritUnderLimit.SelectedValue > crit)
                {
                    crit = (int)this.cmbCritUnderLimit.SelectedValue;
                }
            }

            result.Critical = crit;

            return result;
        }
        #endregion

        //この辺は、IRC送信絡み
        /// <summary>
        /// 最後に生成されたダイスの値を保持するとこ
        /// </summary>
        private DiceRoll _lastCreateDice;

        #region btnSendIRCForRoll_Click / ダイスを発言する
        /// <summary>
        /// ダイスを発言する
        /// </summary>
        private void btnSendIRCForRoll_Click(object sender, EventArgs e)
        {
            if (!this.statusControl.IsChecked)
            {
                if (MessageBox.Show("技能が選択されていません。ダイスを発言しますか？", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                {
                    return;
                }
            }

            if (this._lastCreateDice.Dice > 0 && this.txtDicePriveFormat.TextLength > 0)
            {
                DiceRoll roleDice = CreateAdjustedUnderLimitCriticalDice();

                //IRCにダイスの発言
                //configにある書式でやる
                string message = String.Format(this.txtDicePriveFormat.Text,
                    roleDice.Dice,
                    roleDice.Critical,
                    roleDice.Bonus);

                this.SendIRC(message);
            }
        } 
        #endregion

        #region btnCreateEffectText_Click / コンボ名を作成する
        /// <summary>
        /// コンボ名を作成する
        /// </summary>
        private void btnCreateEffectText_Click(object sender, EventArgs e)
        {
            //エフェクトのコンボ名の設定
            StringBuilder effectCombo = new StringBuilder();

            if (this.txtEffectFormat.TextLength > 0 && this.txtEffectComboSeparater.TextLength > 0)
            {
                foreach (string effectName in this.effectControl.SelectedEffectNames)
                {
                    if (!String.IsNullOrEmpty(effectName) && effectName[0] != '@')
                    {
                        effectCombo.AppendFormat(this.txtEffectFormat.Text, effectName);
                        effectCombo.Append(this.txtEffectComboSeparater.Text);
                    }
                }

                if (effectCombo.Length > 0)
                {
                    effectCombo.Length -= this.txtEffectComboSeparater.TextLength;
                }
            }

            this.txtEffectCombo.Text = effectCombo.ToString();
        } 
        #endregion

        #region btnSendIRCForCombo_Click / コンボを発言する
        /// <summary>
        /// コンボを発言する
        /// </summary>
        private void btnSendIRCForCombo_Click(object sender, EventArgs e)
        {
            if (this.txtEffectCombo.TextLength > 0)
            {
                this.SendIRC(this.txtEffectCombo.Text);
            }
        } 
        #endregion

        #region btnSendIRCForComboDice_Click / コンボに続けて、ダイスを発言する
        /// <summary>
        /// コンボに続けて、ダイスを発言する
        /// </summary>
        private void btnSendIRCForComboDice_Click(object sender, EventArgs e)
        {
            this.btnSendIRCForCombo.PerformClick();
            this.btnSendIRCForRoll.PerformClick();
        } 
        #endregion

        #region SendIRC / IRCにメッセージを送る
        /// <summary>
        /// IRCにメッセージを送る
        /// </summary>
        /// <param name="message">送信するメッセージ</param>
        private void SendIRC(string message)
        {
            string serverName = (string)this.cmbConnectedServers.SelectedItem;
            int serverID = this.chocoa.FindServerID(serverName);
            string channel = (string)this.cmbChannels.SelectedItem;

            this.chocoa.SendPrivateMessage(serverID, channel, message);
        } 
        #endregion
    }
}
