﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace DoubleX3rd
{
    /// <summary>
    /// 肉体その他の４値のコントロール
    /// </summary>
    [DefaultEvent("DiceRollChanged")]
    public partial class StatusControl : UserControl
    {
        #region constructor
        /// <summary>
        /// StatusControl の新しいインスタンスを生成します。
        /// </summary>
        public StatusControl()
        {
            InitializeComponent();
        }
        #endregion

        #region TextBoxMapping / 肉体とかのテキストボックスと、スキルリストのマップ。手抜き用
        /// <summary>
        /// 肉体とかのテキストボックスと、スキルリストのマップ。手抜き用
        /// </summary>
        private Dictionary<TextBox, KeyValuePair<TextBox, SkillListControl>> TextBoxMapping
        {
            get
            {
                Dictionary<TextBox, KeyValuePair<TextBox, SkillListControl>> dic = new Dictionary<TextBox, KeyValuePair<TextBox, SkillListControl>>();

                dic.Add(this.txtBody, new KeyValuePair<TextBox, SkillListControl>(this.txtSelectedBodyRoll, this.ctlSkillBody));
                dic.Add(this.txtSense, new KeyValuePair<TextBox, SkillListControl>(this.txtSelectedSenseRoll, this.ctlSkillSense));
                dic.Add(this.txtMental, new KeyValuePair<TextBox, SkillListControl>(this.txtSelectedMentalRoll, this.ctlSkillMental));
                dic.Add(this.txtSociety, new KeyValuePair<TextBox, SkillListControl>(this.txtSelectedSocietyRoll, this.ctlSkillSociety));

                return dic;
            }
        }
        #endregion

        #region static GetDice / ダイスの数を数える汎用メソッド。手抜き
        /// <summary>
        /// ダイスの数を数える汎用メソッド。手抜き
        /// </summary>
        private static DiceRoll GetDice(TextBox valueBox, SkillListControl skillListControl)
        {
            return GetDice(valueBox, skillListControl.GetCheckedSkillLevel());
        }

        /// <summary>
        /// ダイスの数を数える汎用メソッド。手抜き
        /// </summary>
        private static DiceRoll GetDice(TextBox valueBox, int addSkillLevel)
        {
            DiceRoll result = new DiceRoll();

            int dice = 0;

            if (Int32.TryParse(valueBox.Text, out dice))
            {
                result.Dice = dice;
            }

            result.Critical = 10;
            result.Bonus = addSkillLevel;

            return result;
        } 
        #endregion

        #region OnLoad
        /// <summary>
        /// OnLoad
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            //せっとしといて
            Dictionary<TextBox, KeyValuePair<TextBox, SkillListControl>> dic = this.TextBoxMapping;

            foreach (TextBox targetStatus in dic.Keys)
            {
                dic[targetStatus].Key.Text = GetDice(targetStatus, dic[targetStatus].Value).ToString();
            }
        }
        #endregion

        #region 能力値の入力とか
        /// <summary>
        /// 入力とか
        /// </summary>
        private void txt_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                TextBox textBox = (TextBox)sender;

                if (!this.IsValidTextBox(textBox))
                {
                    textBox.Text = "0";
                }

                this.OnStatusValueChanged(textBox);
                
                e.Handled = true;
            }
        }

        /// <summary>
        /// てぬき
        /// </summary>
        private void txt_Validating(object sender, CancelEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (!this.IsValidTextBox(textBox))
            {
                MessageBox.Show("値が不正です。");
                e.Cancel = true;
            }
            else
            {
                this.OnStatusValueChanged(textBox);
            }
        }

        /// <summary>
        /// 値の変更で、テキストボックスの表示を変えるとこ
        /// </summary>
        /// <param name="textBox">変更されたTextBox</param>
        private void OnStatusValueChanged(TextBox textBox)
        {
            Dictionary<TextBox, KeyValuePair<TextBox, SkillListControl>> dic = this.TextBoxMapping;

            if (dic.ContainsKey(textBox))
            {
                DiceRoll dice = GetDice(textBox, dic[textBox].Value);
                dic[textBox].Key.Text = dice.ToString();

                if (dic[textBox].Value.IsChecked)
                {
                    //選択されている時だけ起こす
                    this.OnDiceRollChanged(new DiceRollChangedEventArgs(dice));
                }
            }
        }

        /// <summary>
        /// 手抜きまくり
        /// </summary>
        private bool IsValidTextBox(TextBox textBox)
        {
            if (textBox != null)
            {
                return (Regex.IsMatch(textBox.Text, @"^[0-9]+$"));
            }

            return false;
        }
        #endregion

        #region ctlSkillBody_CheckedChanged / 肉体のチェックが変わった
        /// <summary>
        /// 肉体のチェックが変わった
        /// </summary>
        private void ctlSkillBody_CheckedChanged(object sender, ItemCheckEventArgs e)
        {
            //ここはまだ選択状態が変わってないので、これじゃないととれない
            DiceRoll dice = GetDice(this.txtBody, this.ctlSkillBody.GetItemSkillLevel(e.Index));

            this.txtSelectedBodyRoll.Text = dice.ToString();
            
            //その他はクリア
            this.ctlSkillSense.ClearCheck();
            this.ctlSkillMental.ClearCheck();
            this.ctlSkillSociety.ClearCheck();

            this.OnDiceRollChanged(new DiceRollChangedEventArgs(dice));
        }
        #endregion

        #region ctlSkillSense_CheckedChanged / 感覚のチェックが変わった
        /// <summary>
        /// 感覚のチェックが変わった
        /// </summary>
        private void ctlSkillSense_CheckedChanged(object sender, ItemCheckEventArgs e)
        {
            DiceRoll dice = GetDice(this.txtSense, this.ctlSkillSense.GetItemSkillLevel(e.Index));

            this.txtSelectedSenseRoll.Text = dice.ToString();

            //その他はクリア
            this.ctlSkillBody.ClearCheck();
            this.ctlSkillMental.ClearCheck();
            this.ctlSkillSociety.ClearCheck();

            this.OnDiceRollChanged(new DiceRollChangedEventArgs(dice));
        }
        #endregion

        #region ctlSkillMental_CheckedChanged / 精神のチェックが変わった
        /// <summary>
        /// 精神のチェックが変わった
        /// </summary>
        private void ctlSkillMental_CheckedChanged(object sender, ItemCheckEventArgs e)
        {
            DiceRoll dice = GetDice(this.txtMental, this.ctlSkillMental.GetItemSkillLevel(e.Index));

            this.txtSelectedMentalRoll.Text = dice.ToString();

            //その他はクリア
            this.ctlSkillBody.ClearCheck();
            this.ctlSkillSense.ClearCheck();
            this.ctlSkillSociety.ClearCheck();

            this.OnDiceRollChanged(new DiceRollChangedEventArgs(dice));
        }
        #endregion

        #region ctlSkillSociety_CheckedChanged / 社会のチェックが変わった
        /// <summary>
        /// 社会のチェックが変わった
        /// </summary>
        private void ctlSkillSociety_CheckedChanged(object sender, ItemCheckEventArgs e)
        {
            DiceRoll dice = GetDice(this.txtSociety, this.ctlSkillSociety.GetItemSkillLevel(e.Index));

            this.txtSelectedSocietyRoll.Text = dice.ToString();

            //その他はクリア
            this.ctlSkillBody.ClearCheck();
            this.ctlSkillSense.ClearCheck();
            this.ctlSkillMental.ClearCheck();

            this.OnDiceRollChanged(new DiceRollChangedEventArgs(dice));
        }
        #endregion

        #region OnDiceRollChanged / DiceRollChanged イベントを発生させます。
        
        /// <summary>
        /// ダイスロールの値が変更された時に発生します。
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Description("ダイスロールの値が変更された時に発生します。")]
        public event DiceRollChangedEventHandler DiceRollChanged;

        /// <summary>
        /// <see cref="DiceRollChanged"/> イベントを発生させます。
        /// </summary>
        private void OnDiceRollChanged(DiceRollChangedEventArgs e)
        {
            this.SelectedDiceRoll = e.DiceRoll;

            if (this.DiceRollChanged != null)
            {
                this.DiceRollChanged(this, e);
            }
        } 
        #endregion

        //設定の変更とか
        #region 肉体のステータスとスキル
        /// <summary>
        /// 肉体を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public int StatusBody
        {
            get
            {
                return (this.txtBody.TextLength == 0) ? 0 : Convert.ToInt32(this.txtBody.Text);
            }
            set
            {
                this.txtBody.Text = value.ToString();
            }
        }

        /// <summary>
        /// 肉体の技能を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public SkillListControl.SkillListItem[] SkillBody
        {
            get
            {
                return this.ctlSkillBody.GetAllSkillListItem();
            }
            set
            {
                this.ctlSkillBody.Clear();

                foreach (SkillListControl.SkillListItem item in value)
                {
                    this.ctlSkillBody.AddSkill(item.SkillName, item.SkillLevel);
                }

                this.OnStatusValueChanged(this.txtBody);
            }
        } 
        #endregion

        #region 感覚のステータスとスキル
        /// <summary>
        /// 感覚を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public int StatusSense
        {
            get
            {
                return (this.txtSense.TextLength == 0) ? 0 : Convert.ToInt32(this.txtSense.Text);
            }
            set
            {
                this.txtSense.Text = value.ToString();
            }
        }

        /// <summary>
        /// 感覚の技能を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public SkillListControl.SkillListItem[] SkillSense
        {
            get
            {
                return this.ctlSkillSense.GetAllSkillListItem();
            }
            set
            {
                this.ctlSkillSense.Clear();

                foreach (SkillListControl.SkillListItem item in value)
                {
                    this.ctlSkillSense.AddSkill(item.SkillName, item.SkillLevel);
                }

                this.OnStatusValueChanged(this.txtSense);
            }
        } 
        #endregion

        #region 精神のステータスとスキル
        /// <summary>
        /// 精神を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public int StatusMental
        {
            get
            {
                return (this.txtMental.TextLength == 0) ? 0 : Convert.ToInt32(this.txtMental.Text);
            }
            set
            {
                this.txtMental.Text = value.ToString();
            }
        }

        /// <summary>
        /// 精神の技能を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public SkillListControl.SkillListItem[] SkillMental
        {
            get
            {
                return this.ctlSkillMental.GetAllSkillListItem();
            }
            set
            {
                this.ctlSkillMental.Clear();

                foreach (SkillListControl.SkillListItem item in value)
                {
                    this.ctlSkillMental.AddSkill(item.SkillName, item.SkillLevel);
                }

                this.OnStatusValueChanged(this.txtMental);
            }
        }
        #endregion

        #region 社会のステータスとスキル
        /// <summary>
        /// 社会を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public int StatusSociety
        {
            get
            {
                return (this.txtSociety.TextLength == 0) ? 0 : Convert.ToInt32(this.txtSociety.Text);
            }
            set
            {
                this.txtSociety.Text = value.ToString();
            }
        }

        /// <summary>
        /// 精神の技能を取得、または設定します。
        /// </summary>
        [Browsable(false)]
        public SkillListControl.SkillListItem[] SkillSociety
        {
            get
            {
                return this.ctlSkillSociety.GetAllSkillListItem();
            }
            set
            {
                this.ctlSkillSociety.Clear();

                foreach (SkillListControl.SkillListItem item in value)
                {
                    this.ctlSkillSociety.AddSkill(item.SkillName, item.SkillLevel);
                }

                this.OnStatusValueChanged(this.txtSociety);
            }
        }
        #endregion

        #region SelectedDiceRoll / 選択されているダイスを取得します。
        /// <summary>
        /// 選択されているダイスを取得します。
        /// </summary>
        [Browsable(false)]
        public DiceRoll SelectedDiceRoll
        {
            get;
            private set;
        } 
        #endregion

        #region IsChecked / チェックされているか
        /// <summary>
        /// チェックされているか
        /// </summary>
        [Browsable(false)]
        public bool IsChecked
        {
            get
            {
                foreach (SkillListControl item in new SkillListControl[] {
                    this.ctlSkillBody,
                    this.ctlSkillMental,
                    this.ctlSkillSociety,
                    this.ctlSkillSense,
                })
                {
                    if (item.IsChecked)
                    {
                        return true;
                    }
                }

                return false;
            }
        }
        #endregion
    }
}
